@@ -27,6 +27,8 @@ class ACKs(Enum):
     greeting = b"PJ_OK"
     pj_ack = b"PJACK"
     pj_req = b"PJREQ"
+    install_acks = b"IN"
+    hdmi_ack = b"IS"
 
 
 class InputModes(Enum):
@@ -42,7 +44,6 @@ class PowerModes(Enum):
     """
     Powermodes on/off: 1/0
     """
-
     off = b"0"
     on = b"1"
 
@@ -64,9 +65,10 @@ class PictureModes(Enum):
     cinema = b"01"
     natural = b"03"
     hdr = b"04"
-    # unsupported
-    THX = b"06"
+    hdr10 = b"04"
+    thx = b"06" # unsupported
     frame_adapt_hdr = b"0B"
+    frame_adapt_hdr1 = b"0B"
     user1 = b"0C"
     user2 = b"0D"
     user3 = b"0E"
@@ -75,15 +77,24 @@ class PictureModes(Enum):
     user6 = b"11"
     hlg = b"14"
     hdr_plus = b"15"
+    hdr10_plus = b"15"
     pana_pq = b"16"
+    filmmaker = b"17" # requires firmware 2.0
+    frame_adapt_hdr2 = b"18" # requires firmware 2.0
+    frame_adapt_hdr3 = b"19" # requires firmware 2.0
 
 
-class MemoryModes(Enum):
-    memory1 = b"0"
-    memory2 = b"1"
-    memory3 = b"2"
-    memory4 = b"3"
-    memory5 = b"4"
+class InstallationModes(Enum):
+    mode1 = b"0"
+    mode2 = b"1"
+    mode3 = b"2"
+    mode4 = b"3"
+    mode5 = b"4"
+    mode6 = b"5"
+    mode7 = b"6"
+    mode8 = b"7"
+    mode9 = b"8"
+    mode10 = b"9"
 
 
 class LowLatencyModes(Enum):
@@ -91,7 +102,6 @@ class LowLatencyModes(Enum):
     Low latency requires certain options turned off first
     It is not a function. Will not work without disabling
     CMD, dynamic ctrl, others
-    Use the low latency provided function instead
     """
 
     off = b"0"
@@ -99,14 +109,12 @@ class LowLatencyModes(Enum):
 
 
 class MotionEnhanceModes(Enum):
-
     off = b"0"
     low = b"1"
     high = b"2"
 
 
 class GraphicModeModes(Enum):
-
     standard = b"0"
     hires1 = b"1"
     hires2 = b"2"
@@ -149,6 +157,7 @@ class LaserDimModes(Enum):
     off = b"0"
     auto1 = b"1"
     auto2 = b"2"
+    auto3 = b"3" # requires firmware 2.0
 
 
 class EnhanceModes(Enum):
@@ -180,8 +189,19 @@ class AnamorphicModes(Enum):
     a = b"1"
     b = b"2"
     c = b"3"
+    d = b"4"
 
+class ColorSpaceModes(Enum):
+    auto = b"0"
+    YCbCr444 = b"1"
+    YCbCr422 = b"2"
+    RGB = b"3"
 
+class InputLevel(Enum):
+    standard = b"0"
+    enhanced = b"1"
+    superwhite = b"2"
+    auto = b"3"
 class EshiftModes(Enum):
     off = b"0"
     on = b"1"
@@ -198,11 +218,11 @@ class Commands(Enum):
     # power commands
     power = b"PW", PowerModes, ACKs.power_ack
 
-    # lens memory commands
-    memory = b"INML", MemoryModes, ACKs.lens_ack
+    # lens memory /installation mode commands
+    installation_mode = b"INML", InstallationModes, ACKs.lens_ack
 
     # input commands
-    input = b"IP", InputModes, ACKs.input_ack
+    input_mode = b"IP", InputModes, ACKs.input_ack
 
     # status commands - Reference: ?
     # These should not be used directly
@@ -212,6 +232,12 @@ class Commands(Enum):
 
     # picture mode commands
     picture_mode = b"PMPM", PictureModes, ACKs.picture_ack
+    
+    # Color modes
+    color_mode = b"ISHS", ColorSpaceModes, ACKs.hdmi_ack
+
+    # input_level like 0-255
+    input_level = b"ISIL", InputLevel, ACKs.hdmi_ack
 
     # low latency enable/disable
     low_latency = b"PMLL", LowLatencyModes, ACKs.picture_ack
@@ -232,7 +258,7 @@ class Commands(Enum):
     menu = b"RC73", MenuModes, ACKs.menu_ack
 
     # NZ Series Laser Dimming commands
-    laser_dim = b"PMDC", LaserDimModes, ACKs.picture_ack
+    laser_mode = b"PMDC", LaserDimModes, ACKs.picture_ack
 
     # Lens Aperture commands
     aperture = b"PMDI", ApertureModes, ACKs.picture_ack
@@ -242,4 +268,4 @@ class Commands(Enum):
     anamorphic = b"INVS", AnamorphicModes, ACKs.lens_ack
 
     # e-shift
-    eshift = b"PMUS", EshiftModes, ACKs.picture_ack
+    eshift_mode = b"PMUS", EshiftModes, ACKs.picture_ack
