@@ -3,10 +3,9 @@ Implements the JVC protocol
 """
 
 import logging
-from typing import Final, Union, Awaitable, Callable
+from typing import Final, Union
 import asyncio
-from jvc_projector.optimal_settings import hdr_film, hdr_game, sdr_film, sdr_game
-from jvc_projector.commands import ACKs, Footer, Header, Commands, PowerStates, Enum
+from jvc_projector.commands import InputLevel, ColorSpaceModes,EshiftModes ,ACKs, Footer, Header, Commands, PowerStates, PictureModes, InstallationModes, InputModes, LaserDimModes, Enum, LowLatencyModes
 
 
 class JVCProjector:
@@ -18,9 +17,7 @@ class JVCProjector:
         password: str = "",
         # Can supply a logger object. It can hook into the HA logger
         logger: logging.Logger = logging.getLogger(__name__),
-        connection_lost_callback: Callable[[], Awaitable[None]] = None,
         loop: asyncio.AbstractEventLoop = None,
-        update_callback: Callable[[str], None] = None,
         port: int = 20554,
         connect_timeout: int = 10,
     ):
@@ -30,82 +27,56 @@ class JVCProjector:
         self.password = password
         self.connect_timeout: int = connect_timeout
         self.logger = logger
-        self._update_callback = update_callback
-        # use the provided loop or get current one. Otherwise make one
-        try:
-            self._loop = loop or asyncio.get_running_loop()
-        except RuntimeError:
-            self._loop = asyncio.new_event_loop()
         self._lock = asyncio.Lock()
         # Const values
         self.PJ_OK: Final = ACKs.greeting.value
         self.PJ_ACK: Final = ACKs.pj_ack.value
         self.PJ_REQ: Final = ACKs.pj_req.value
-        self._connection_lost_callback = connection_lost_callback
-        self._closed = False
-        self._retry_interval = 1
-        self._closing = False
-        self._halted = False
         self.reader: asyncio.StreamReader = None
         self.writer: asyncio.StreamWriter = None
+        self.command_read_timeout = 3
 
     async def async_open_connection(self) -> bool:
         """Open a connection"""
-        assert self.port >= 0, f"Port must be greater than 0: {self.port}"
-        assert self.host != "", f"Host must not be empty: {self.host}"
+        self.logger.debug("Starting open connection")
         msg, success = await self.reconnect()
+
         if not success:
             self.logger.error(msg)
 
         return success
 
     async def reconnect(self):
-        """Initiate keep-alive connection"""
+        """Initiate keep-alive connection. This should handle any error and reconnect eventually."""
         while True:
             try:
-                if self._halted:
-                    await asyncio.sleep(2)
-
-                self.logger.debug(
+                if self.writer is not None:
+                    self.logger.debug("Closing writer")
+                    self.writer.close()
+                    await self.writer.wait_closed()
+                self.logger.info(
                     "Connecting to JVC Projector: %s:%s", self.host, self.port
                 )
-                async with self._lock:
-                    # transport, protocol = await self._loop.create_connection(
-                    #     lambda: self.protocol, self.host, self.port
-                    # )
-                    self.reader, self.writer = await asyncio.open_connection(
-                        self.host, self.port, loop=self._loop
-                    )
-                    self.logger.debug("Connected to JVC Projector")
-                    # create a reader and writer to do handshake
-                async with self._lock:
-                    self.logger.debug("Handshaking")
-                    result, success = await self._async_handshake()
-                    if not success:
-                        return result, success
-                    self.logger.debug("Handshake complete and we are connected")
-                self._reset_retry_interval()
+                cor = asyncio.open_connection(self.host, self.port)
+                # wait for 10 sec to connect
+                self.reader, self.writer = await asyncio.wait_for(cor, 10)
+                self.logger.info("Connected to JVC Projector")
+                # create a reader and writer to do handshake
+                self.logger.debug("Handshaking")
+                result, success = await self._async_handshake()
+                if not success:
+                    return result, success
+                self.logger.debug("Handshake complete and we are connected")
                 return "Connection done", True
 
             # includes conn refused
             except OSError as err:
-                self._increase_retry_interval()
-                interval = self._get_retry_interval()
-                self.logger.warning(
-                    "Connecting failed, retrying in %i seconds", interval
-                )
-                if not self._closing:
-                    return f"Connection failed: {err}", False
-                await asyncio.sleep(interval)
+                self.logger.warning("Connecting failed, retrying in 2 seconds")
+                self.logger.debug(err)
+                await asyncio.sleep(2)
             except asyncio.TimeoutError:
-                self._increase_retry_interval()
-                interval = self._get_retry_interval()
-                self.logger.warning(
-                    "Connection timed out, retrying in %i seconds", interval
-                )
-                if not self._closing:
-                    return "Connection timed out", False
-                await asyncio.sleep(interval)
+                self.logger.warning("Connection timed out, retrying in 2 seconds")
+                await asyncio.sleep(2)
 
     async def _async_handshake(self) -> tuple[str, bool]:
         """
@@ -116,6 +87,7 @@ class JVCProjector:
         """
         if self.password:
             pj_req = self.PJ_REQ + f"_{self.password}".encode()
+            self.logger.debug("connecting with password hunter2")
         else:
             pj_req = self.PJ_REQ
 
@@ -124,54 +96,27 @@ class JVCProjector:
         self.logger.debug(msg_pjok)
         if msg_pjok != self.PJ_OK:
             result = f"Projector did not reply with correct PJ_OK greeting: {msg_pjok}"
-            success = False
-
-            return result, success
+            self.logger.error(result)
+            return result, False
 
         # try sending PJREQ, if there's an error, raise exception
         try:
             self.writer.write(pj_req)
             await self.writer.drain()
         except asyncio.TimeoutError as err:
-            return f"Timeout sending PJREQ {err}", False
+            result = f"Timeout sending PJREQ {err}"
+            self.logger.error(result)
+            return result, False
 
         # see if we receive PJACK, if not, raise exception
         msg_pjack = await self.reader.read(len(self.PJ_ACK))
         if msg_pjack != self.PJ_ACK:
-            return f"Exception with PJACK: {msg_pjack}", False
-
+            result = f"Exception with PJACK: {msg_pjack}"
+            self.logger.error(result)
+            return result, False
+        self.logger.debug("Handshake successful")
         return "ok", True
 
-    async def connection_lost(self):
-        """restart connection"""
-        if not self._closing:
-            await self.reconnect()
-
-    def _get_retry_interval(self):
-        return self._retry_interval
-
-    def _reset_retry_interval(self):
-        self._retry_interval = 1
-
-    def _increase_retry_interval(self):
-        self._retry_interval = min(300, 1.5 * self._retry_interval)
-
-    def close(self):
-        # TODO: maybe implement these
-        """Close the AVR device connection and don't try to reconnect."""
-        self.logger.debug("Closing connection to AVR")
-        self._closing = True
-
-    def halt(self):
-        """Close the AVR device connection and wait for a resume() request."""
-        self.logger.warning("Halting connection to AVR")
-        self._halted = True
-
-    def resume(self):
-        """Resume the AVR device connection if we have been halted."""
-        self.logger.warning("Resuming connection to AVR")
-        self._halted = False
-
     async def _async_send_command(
         self,
         send_command: Union[list[bytes], bytes],
@@ -195,6 +140,11 @@ class JVCProjector:
             )
         """
 
+        if self.writer is None:
+            self.logger.error("Connection lost. Restarting")
+
+            await self.reconnect()
+
         # Check commands
         if command_type == Header.reference.value:
             result, success = await self._async_do_command(
@@ -204,7 +154,6 @@ class JVCProjector:
             return result, success
 
         if isinstance(send_command, list):
-
             for cmd in send_command:
                 cons_command, ack = await self._async_construct_command(
                     cmd, command_type
@@ -243,8 +192,9 @@ class JVCProjector:
         ack: bytes,
         command_type: bytes = b"!",
     ) -> tuple[str, bool]:
-        async with self._lock:
-            self.logger.debug("_do_command sending command: %s", command)
+        retry_count = 0
+        while retry_count < 5:
+            self.logger.debug("do_command sending command: %s", command)
             # send the command
             self.writer.write(command)
             try:
@@ -252,10 +202,14 @@ class JVCProjector:
             except ConnectionError as err:
                 # reaching this means the writer was closed somewhere
                 self.logger.error(err)
+                self.logger.debug("Restarting connection")
                 # restart the connection
-                await self.connection_lost()
-                self.writer.write(command)
-                await self.writer.drain()
+
+                await self.reconnect()
+                self.logger.debug("Sending command again")
+                # restart the loop
+                retry_count += 1
+                continue
 
             # if we send a command that returns info, the projector will send
             # an ack, followed by the actual message. Check to see if the ack sent by
@@ -263,43 +217,73 @@ class JVCProjector:
             ack_value = (
                 Header.ack.value + Header.pj_unit.value + ack + Footer.close.value
             )
-            self.logger.debug("ack_value: %s", ack_value)
+            self.logger.debug("constructed ack_value: %s", ack_value)
 
             # Receive the acknowledgement from PJ
             try:
-                received_ack = await self.reader.read(len(ack_value))
+                # seems like certain commands timeout when PJ is off
+                received_ack = await asyncio.wait_for(
+                    self.reader.readline(), timeout=self.command_read_timeout
+                )
             except asyncio.TimeoutError:
+                # LL is used in async_update() and I don't want to spam HA logs so we skip
+                # if not command == b"?\x89\x01PMLL\n":
                 # Sometimes if you send a command that is greyed out, the PJ will just hang
-                result = "Connection timed out. Command is probably not allowed to run at this time."
-                self.logger.error(result)
-                return result, False
+                self.logger.error(
+                    "Connection timed out. Command %s is probably not allowed to run at this time.",
+                    command,
+                )
+                self.logger.debug("restarting connection")
+
+                await self.reconnect()
+                retry_count += 1
+                continue
+
             except ConnectionRefusedError:
                 self.logger.error("Connection Refused when getting ack")
-                return "Connection Refused", False
+                self.logger.debug("restarting connection")
 
-            self.logger.debug("received_ack: %s", received_ack)
+                await self.reconnect()
+                retry_count += 1
+                continue
+
+            self.logger.debug("received ack from PJ: %s", received_ack)
+
+            # This will probably never happen since we are handling timeouts now
+            if received_ack == b"":
+                self.logger.error("Got a blank ack. Restarting connection")
+
+                await self.reconnect()
+                retry_count += 1
+                continue
 
             # get the ack for operation
             if received_ack == ack_value and command_type == Header.operation.value:
-                self.logger.debug("result: %s", received_ack)
                 return received_ack, True
 
             # if we got what we expect and this is a reference,
             # receive the data we requested
             if received_ack == ack_value and command_type == Header.reference.value:
-                message = await self.reader.read(1024)
-                self.logger.debug("result: %s, %s", received_ack, message)
+                message = await self.reader.readline()
+                self.logger.debug("received message from PJ: %s", message)
 
                 return message, True
 
             # Otherwise, it failed
             # Because this now reuses a connection, reaching this stage means catastrophic failure, or HA running as usual :)
-            result = "Unexpected ack received from PJ after sending a command. Perhaps a command got cancelled because a new connection was made."
-            self.logger.error(result)
-            self.logger.error("received_ack: %s", received_ack)
-            self.logger.error("ack_value: %s", ack_value)
+            self.logger.error(
+                "Recieved ack did not match expected ack: %s != %s",
+                received_ack,
+                ack_value,
+            )
+            # Try to restart connection, if we got here somethihng is out of sync
 
-            return result, False
+            await self.reconnect()
+            retry_count += 1
+            continue
+
+        self.logger.error("retry count for running commands exceeded")
+        return "retry count exceeded", False
 
     async def _async_construct_command(
         self, raw_command: str, command_type: bytes
@@ -404,154 +388,16 @@ class JVCProjector:
         Turns off PJ
         """
         return await self.async_exec_command("power,off")
-
-    async def async_gaming_mode_hdr(self) -> bool:
+    
+    async def _async_replace_headers(self, item: bytes) -> bytes:
         """
-        Sets (opinionated!) optimal HDR gaming settings
+        Will strip all headers and returns the value itself
         """
-        # TODO: use Userx modes for this
-        state = await self.async_get_low_latency_state()
-        await asyncio.sleep(3)
-        # If LL is on, we can turn it off first
-        # TODO: make this more DRY
-        if state:
-            cmds = [
-                "picture_mode, hdr",
-                "enhance, seven",
-                "motion_enhance, off",
-                "graphic_mode, hires1",
-            ]
+        headers = [x.value for x in Header] + [x.value for x in Footer]
+        for header in headers:
+            item = item.replace(header, b"")
 
-            # ll_commands = self._build_mode_commands(cmds)
-            # return ll_commands
-            return await self.async_exec_command(cmds)
-        else:
-            # If LL is off, we can enable these settings
-            cmds = [
-                "picture_mode, hdr",
-                "laser_dim, off",
-                "low_latency, on",
-                "enhance, seven",
-                "motion_enhance, off",
-                "graphic_mode, hires1",
-            ]
-        return await self.async_exec_command(cmds)
-
-    async def async_gaming_mode_sdr(self) -> bool:
-        """
-        Sets (opinionated!) optimal gaming settings
-        """
-        # task = asyncio.create_task(self.async_get_low_latency_state())
-        state = await self.async_get_low_latency_state()
-        # todo: fix this, its a concurrency issue
-        await asyncio.sleep(3)
-        # If LL is on, we can turn it off first
-
-        if state is None:
-            return "Error getting low latency state"
-
-        if state:
-            cmds = [
-                "enhance, seven",
-                "motion_enhance, off",
-                "graphic_mode, hires1",
-            ]
-
-            # ll_commands = self._build_mode_commands(cmds)
-            # return ll_commands
-            return await self.async_exec_command(cmds)
-        elif state is False:
-            # If LL is off, we can enable these settings
-            cmds = [
-                "laser_dim, off",
-                "low_latency, on",
-                "enhance, seven",
-                "motion_enhance, off",
-                "graphic_mode, hires1",
-            ]
-        return await self.async_exec_command(cmds)
-
-    async def async_hdr_picture_mode(self) -> bool:
-        """
-        Sets (opinionated!) optimal HDR film settings
-
-        fyi: With the API turn on or off LL first otherwise it will be greyed out in the menu
-        so PJ socket will time out
-        """
-        state = await self.async_get_low_latency_state()
-        await asyncio.sleep(3)
-        # If LL is on, we can turn it off first
-        # TODO: make this more DRY
-        if state:
-            cmds = [
-                "low_latency, off",
-                "picture_mode, frame_adapt_hdr",
-                "laser_dim, auto1",
-                "enhance, seven",
-                "motion_enhance, low",
-                "graphic_mode, hires1",
-            ]
-
-            # ll_commands = self._build_mode_commands(cmds)
-            # return ll_commands
-            return await self.async_exec_command(cmds)
-        else:
-            # If LL is off, we can enable these settings
-            cmds = [
-                "picture_mode, frame_adapt_hdr",
-                "laser_dim, off",
-                "enhance, seven",
-                "motion_enhance, low",
-                "graphic_mode, hires1",
-            ]
-
-        return await self.async_exec_command(cmds)
-
-    async def async_sdr_picture_mode(self) -> bool:
-        """
-        Sets (opinionated) optimal sdr film settings
-        """
-        state = await self.async_get_low_latency_state()
-        await asyncio.sleep(3)
-        # If LL is on, we need to turn it off
-        # TODO: make this more DRY
-        if state:
-            cmds = [
-                "low_latency, off",
-                "laser_dim, auto1",
-                "enhance, seven",
-                "motion_enhance, low",
-                "graphic_mode, hires1",
-            ]
-
-            # ll_commands = self._build_mode_commands(cmds)
-            # return ll_commands
-            return await self.async_exec_command(cmds)
-        else:
-            # If LL is off, we can enable these settings
-            cmds = [
-                "laser_dim, auto1",
-                "enhance, seven",
-                "motion_enhance, low",
-                "graphic_mode, hires1",
-            ]
-        return await self.async_exec_command(cmds)
-
-    # def _build_mode_commands(self, commands: list) -> list[str]:
-    #     modes = self._check_low_latency()
-    #     if modes == []:
-    #         return modes
-
-    #     ll_cmds = []
-    #     for mode in modes:
-    #         if "laser_dim" in mode:
-    #             ll_cmds.append("laser_dim, off")
-    #         if "picture_mode" in mode:
-    #             ll_cmds.append("picture_mode, hdr")
-
-    #     ll_cmds.extend(commands)
-
-    #     return ll_cmds
+        return item
 
     async def _async_do_reference_op(self, command: str, ack: ACKs) -> tuple[str, bool]:
         cmd = (
@@ -580,65 +426,75 @@ class JVCProjector:
     async def async_get_low_latency_state(self) -> bool:
         """
         Get the current state of LL
-
-        None if there was an error
         """
-        state, success = await self._async_do_reference_op(
+        state, _ = await self._async_do_reference_op(
             "low_latency", ACKs.picture_ack
         )
-        # LL is off, could be disabled
-        if not success:
-            return None
-        if state == b"PM0":
-            return False
 
-        return True
-
-    # async def _async_check_low_latency(self) -> list[str]:
-    #     """
-    #     Infer if Low Latency is disabled or not otherwise commands will hang
-
-    #     Returns enabled modes that have to be disabled first
-    #     """
-    #     enabled_modes = []
-
-    #     state = await self.get_low_latency_state()
-    #     # LL is off, could be disabled
-    #     if state:
-    #         state, success = await self._async_do_reference_op(
-    #             "laser_dim", ACKs.picture_ack
-    #         )
-    #         if success:
-    #             if state != b"PM0":
-    #                 self.logger.debug("Laser dimming is enabled")
-    #                 enabled_modes.append("laser_dim")
-    #         else:
-    #             return "Error", state
-    #         # see if its hdr10+ or frame adapt
-    #         state, success = await self._async_do_reference_op(
-    #             "picture_mode", ACKs.picture_ack
-    #         )
-    #         if success:
-    #             if state in [b"PM0B", b"PM15"]:
-    #                 self.logger.debug("Locked HDR is enabled")
-    #                 enabled_modes.append("picture_mode")
-    #         else:
-    #             return "Error", state
-
-    #     self.logger.debug("Low Latency check: %s", success)
-
-    #     return enabled_modes
-
-    async def _async_replace_headers(self, item: bytes) -> bytes:
+        return LowLatencyModes(state.replace(ACKs.picture_ack.value, b"")).name
+    
+    async def async_get_picture_mode(self) -> str:
         """
-        Will strip all headers and returns the value itself
+        Get the current picture mode as str -> user1, natural
         """
-        self.logger.debug("replacing headers for %s of type %s", item, type(item))
-        headers = [x.value for x in Header] + [x.value for x in Footer]
-        for header in headers:
-            item = item.replace(header, b"")
-
-        return item
+        state, _ = await self._async_do_reference_op(
+            "picture_mode", ACKs.picture_ack
+        )
+        return PictureModes(state.replace(ACKs.picture_ack.value, b"")).name
+    
+    async def async_get_install_mode(self) -> str:
+        """
+        Get the current install mode as str
+        """
+        state, _ = await self._async_do_reference_op(
+            "installation_mode", ACKs.install_acks
+        )
+        return InstallationModes(state.replace(ACKs.install_acks.value, b"")).name
+    
+    async def async_get_input_mode(self) -> str:
+        """
+        Get the current input mode
+        """
+        state, _ = await self._async_do_reference_op(
+            "input_mode", ACKs.input_ack
+        )
+        return InputModes(state.replace(ACKs.input_ack.value, b"")).name
+    
+    async def async_get_laser_mode(self) -> str:
+        """
+        Get the current laser mode
+        """
+        state, _ = await self._async_do_reference_op(
+            "laser_mode", ACKs.picture_ack
+        )
+        return LaserDimModes(state.replace(ACKs.picture_ack.value, b"")).name
+    
+    async def async_get_eshift_mode(self) -> str:
+        """
+        Get the current eshift mode
+        """
+        state, _ = await self._async_do_reference_op(
+            "eshift_mode", ACKs.picture_ack
+        )
+        return EshiftModes(state.replace(ACKs.picture_ack.value, b"")).name
+   
+    async def async_get_color_mode(self) -> str:
+        """
+        Get the current color mode
+        """
+        state, _ = await self._async_do_reference_op(
+            "color_mode", ACKs.hdmi_ack
+        )
+        return ColorSpaceModes(state.replace(ACKs.hdmi_ack.value, b"")).name
+    
+    async def async_get_input_level(self) -> str:
+        """
+        Get the current input level
+        """
+        state, _ = await self._async_do_reference_op(
+            "input_level", ACKs.hdmi_ack
+        )
+        return InputLevel(state.replace(ACKs.hdmi_ack.value, b"")).name
 
     async def _async_get_power_state(self) -> str:
         """
@@ -676,8 +532,14 @@ class JVCProjector:
         """
         True if the current state is on|reserved
         """
-        pw_status = [PowerStates.on.name, PowerStates.reserved.name]
+        pw_status = [PowerStates.on.name]
         return await self._async_get_power_state() in pw_status
+    
+    async def async_is_ll_on(self) -> bool:
+        """
+        True if LL mode is on
+        """
+        return await self.async_get_low_latency_state() == LowLatencyModes.on.name
 
     def print_commands(self) -> str:
         """
